// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

part of prompt.framework;

abstract class Widget<T extends dynamic> {
  int _initLineCount = 0;

  /// Creates the mutable state for this widget
  @protected
  @factory
  State createState();

  State pipeState(State state) => state;

  T prompt() {
    final state = pipeState(createState())
      .._widget = this
      ..initState();
    _initLineCount = state.context.lineCount;
    state.context.resetLineCount();

    state.build();
    state.context.incrementBuildCount();

    final dynamic out = state.prompt();
    state.context.wipe();
    state.context.erasePreviousLine(_initLineCount);
    state.dispose();

    return out as T;
  }
}
